// *************************************************************************
//
// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
//
// This file is part of the BOUML Uml Toolkit.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// e-mail : bouml@free.fr
// home   : http://bouml.free.fr
//
// *************************************************************************

#include <stdio.h>
#include <qtextstream.h> 

#include "UmlOperation.h"
#include "UmlSettings.h"
#include "JavaSettings.h"
#include "UmlCom.h"
#include "UmlClass.h"
#include "UmlRelation.h"
#include "util.h"

static bool generate_type(const QValueList<UmlParameter> & params,
			  unsigned rank, QTextOStream & f)
{
  if (rank >= params.count())
    return FALSE;
  
  UmlClass::write(f, params[rank].type);
  return TRUE;
}

static bool generate_var(const QValueList<UmlParameter> & params, 
			 unsigned rank, QTextOStream & f)
{
  if (rank >= params.count())
    return FALSE;
  
  f << params[rank].name;
  return TRUE;
}

static void param_error(const QCString & parent, const QCString & name, unsigned rank)
{
  write_trace_header();
  UmlCom::trace(QCString("<tt>        </tt><font face=helvetica><b>while compiling <i>")
		+ parent + "::" + name + "</i> parameter rank " + QCString().setNum(rank)
		+ " does not exist</font></b><br>");
}

QCString UmlOperation::compute_name() {
  QCString get_set_spec = javaNameSpec();
  
  if (! get_set_spec.isEmpty()) {
    UmlClassMember * it;
    
    if ((it = getOf()) == 0)
      it = setOf();
    
    int index;
    QCString s = (it->kind() == aRelation)
      ? ((UmlRelation *) it)->roleName()
      : it->name();
    
    if ((index = get_set_spec.find("${name}")) != -1)
      get_set_spec.replace(index, 7, s);
    else if ((index = get_set_spec.find("${Name}")) != -1)
      get_set_spec.replace(index, 7, capitalize(s));
    else if ((index = s.find("${NAME}")) != -1)
      get_set_spec.replace(index, 7, s.upper());
    
    return get_set_spec;
  }
  else
    return name();
}

// p0 is the beginning of the operation's def
// p point to ${body}
// indent is the one of the operation
static const char * generate_body(QTextOStream & f, QCString indent,
				  const char * body, const char * p)
{
  // get keyword indent
  QCString bindent = indent;
  
  while (*p != '$')
    bindent += *p++;

  if ((body != 0) && (*body != 0)) {
    // output body
    f << bindent;
    
    for (; *body; body += 1) {
      f << *body;
      if (*body == '\n') {
	if (body[1] == 0)
	  break;
	f << bindent;
      }
    }
  }
  f << indent;	// for the }
  
  return p + 7;
}

void UmlOperation::generate(QTextOStream & f, const QCString & cl_stereotype,
			    QCString indent) {
  if (!javaDecl().isEmpty()) {
    const char * p = javaDecl();
    const char * pp = 0;
    const QValueList<UmlParameter> & params = this->params();
    unsigned rank;
    const char * body_indent = strstr(p, "${body}");
    
    if (body_indent != 0) {
      while ((body_indent != p) &&
	     (body_indent[-1] == ' ') || (body_indent[-1] == '\t'))
	body_indent -= 1;
    }
    
    // manage old style indent
    while ((*p == ' ') || (*p == '\t'))
      indent += *p++;
  
    f << indent;
    
    for (;;) {
      if (*p == 0) {
	if (pp == 0)
	  break;
	
	// comment management done
	p = pp;
	pp = 0;
	if (*p == 0)
	  break;
	f << indent;
      }

      if (*p == '\n') {
	f << *p++;
	if (p == body_indent)
	  p = generate_body(f, indent, javaBody(), p);
	else if (*p)
	  f << indent;
      }
      else if ((*p == '{') && 
	       (isAbstract() ||
		(cl_stereotype == "interface") ||
		(cl_stereotype == "@interface"))) {
	f << ";";
	break;
      }
      else if (*p == '@')
	manage_alias(p, f);
      else if (*p != '$') {
	if (p == body_indent)
	  p = generate_body(f, indent, javaBody(), p);
	else
	  f << *p++;
      }
      else if (!strncmp(p, "${comment}", 10))
	manage_comment(p, pp);
      else if (!strncmp(p, "${description}", 14))
	manage_description(p, pp);
      else if (!strncmp(p, "${visibility}", 13)) {
	p += 13;
	generate_visibility(f);
      }
      else if (!strncmp(p, "${final}", 8)) {
	p += 8;
	if (isJavaFinal())
	  f << "final ";
      }
      else if (!strncmp(p, "${static}", 9)) {
	p += 9;
	if (isClassMember())
	  f << "static ";
      }
      else if (!strncmp(p, "${abstract}", 11)) {
	p += 11;
	if (isAbstract() && 
	    (cl_stereotype != "interface") &&
	    (cl_stereotype != "@interface"))
	  f << "abstract ";
      }
      else if (!strncmp(p, "${synchronized}", 15)) {
	p += 15;
	if (isJavaSynchronized())
	  f << "synchronized ";
      }
      else if (!strncmp(p, "${type}", 7)) {
	p += 7;
	UmlClass::write(f, returnType());
      }
      else if (!strncmp(p, "${stereotype}", 13)) {
	p += 13;
	// get/set relation with multiplicity > 1
	f << JavaSettings::relationStereotype(stereotype());
      }
      else if (!strncmp(p, "${name}", 7)) {
	p += 7;
	f << compute_name();
      }
      else if (!strncmp(p, "${(}", 4)) {
	p += 4;
	f << '(';
      }
      else if (!strncmp(p, "${)}", 4)) {
	p += 4;
	f << ')';
      }
      else if (sscanf(p, "${t%u}", &rank) == 1) {
	if (!generate_type(params, rank, f))
	  param_error(parent()->name(), name(), rank);
	p = strchr(p, '}') + 1;
      }
      else if (sscanf(p, "${p%u}", &rank) == 1) {
	if (!generate_var(params, rank, f))
	  param_error(parent()->name(), name(), rank);
	p = strchr(p, '}') + 1;
      }
      else if (!strncmp(p, "${throws}", 9)) {
	p += 9;
	const char * sep;
	const QValueList<UmlTypeSpec> & exceptions = this->exceptions();
	QValueList<UmlTypeSpec>::ConstIterator it;
	
	for (it = exceptions.begin(), sep = " throws "; 
	     it != exceptions.end();
	     ++it, sep = " ") {
	  f << sep;
	  UmlClass::write(f, *it);
	}
      }
      else if (!strncmp(p, "${staticnl}", 11)) {
	p += 11;
	if (isClassMember()) {
	  f << '\n';
	  if (*p)
	    f << indent;
	}
	else
	  f << ' ';
      }
      else if (!strncmp(p, "${body}", 7))
	p = generate_body(f, indent, javaBody(), p);
      else if (!strncmp(p, "${association}", 14)) {
	p += 14;
	
	UmlClassMember * m;
	
	if (((m = getOf()) != 0) && (m->kind() == aRelation))
	  UmlClass::write(f, ((UmlRelation *) m)->association());
	else if (((m = setOf()) != 0) && (m->kind() == aRelation))
	  UmlClass::write(f, ((UmlRelation *) m)->association());
      }
      else if (!strncmp(p, "${@}", 4)) {
	p += 4;
	if (pp != 0)
	  f << "${@}";
	else if (! javaAnnotations().isEmpty()) {
	  pp = p;
	  p = javaAnnotations();
	}
      }
      else
	// strange
	f << *p++;
    }
    
    f << '\n';
  }
}

void UmlOperation::generate_enum_pattern_item(QTextOStream &, int &,
					      const QCString &, QCString) {
  write_trace_header();
  UmlCom::trace("<tt>        </tt><font face=helvetica><b>an <i>enum</i> cannot have operation</b></font><br>");
}

void UmlOperation::generate_enum_pattern_case(QTextOStream &, QCString) {
  // error already signaled
}

void UmlOperation::generate_enum_member(QTextOStream & f, QCString indent) {
  generate(f, "enum", indent);
}
