// *************************************************************************
//
// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
//
// This file is part of the BOUML Uml Toolkit.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// e-mail : bouml@free.fr
// home   : http://bouml.free.fr
//
// *************************************************************************


#ifdef WIN32
#pragma warning (disable: 4150)
#endif

#include <qhbox.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qcombobox.h> 
#include <qpushbutton.h>

#include "Settings.h"
#include "SettingsDialog.h"
#include "UmlPixmap.h"
#include "UmlDesktop.h"

class ComboStates : public QComboBox {
  public:
    ComboStates(QWidget * parent, Uml3States v, bool nodefault);
    ComboStates(QWidget * parent, ClassDrawingMode v, bool nodefault);
    ComboStates(QWidget * parent, DrawingLanguage v, bool nodefault);
    ComboStates(QWidget * parent, ShowContextMode v, bool nodefault);
};

ComboStates::ComboStates(QWidget * parent, Uml3States v, bool nodefault) 
    : QComboBox(FALSE, parent) {
  insertItem(stringify((Uml3States) 0));
  insertItem(stringify((Uml3States) 1));
  // the last value MUST be default
  if (!nodefault)
    insertItem(stringify((Uml3States) 2));
  setCurrentItem(v);
}

ComboStates::ComboStates(QWidget * parent, ClassDrawingMode v, bool nodefault) 
    : QComboBox(FALSE, parent) {
  int i;
  
  for (i = 0; i != (int) DefaultClassDrawingMode; i += 1)
    insertItem(stringify((ClassDrawingMode) i));
  // the last value MUST be default
  if (!nodefault)
    insertItem(stringify((ClassDrawingMode) i));
  setCurrentItem(v);
}

ComboStates::ComboStates(QWidget * parent, DrawingLanguage v, bool nodefault) 
    : QComboBox(FALSE, parent) {
  int i;
  
  for (i = 0; i != (int) DefaultDrawingLanguage; i += 1)
    insertItem(stringify((DrawingLanguage) i));
  // the last value MUST be default
  if (!nodefault)
    insertItem(stringify((DrawingLanguage) i));
  setCurrentItem(v);
}

ComboStates::ComboStates(QWidget * parent, ShowContextMode v, bool nodefault) 
    : QComboBox(FALSE, parent) {
  int i;
  
  for (i = 0; i != (int) DefaultShowContextMode; i += 1)
    insertItem(stringify((ShowContextMode) i));
  // the last value MUST be default
  if (!nodefault)
    insertItem(stringify((ShowContextMode) i));
  setCurrentItem(v);
}

class ComboColor : public QComboBox {
  public:
    ComboColor(QWidget * parent, UmlColor v, bool nodefault);
};

ComboColor::ComboColor(QWidget * parent, UmlColor v, bool nodefault) 
    : QComboBox(FALSE, parent) {
  // the last value MUST be default
  for (int i = 0; i != (int) UmlDefaultColor; i += 1) {
    // use switch to not be dependant on the colors items order and number
    switch (i) {
    case UmlTransparent:
      insertItem("transparent");
      break;
    case UmlWhite:
      insertItem(* WhitePixmap);
      break;
    case UmlLightYellow:
      insertItem(* LightYellowPixmap);
      break;
    case UmlYellow:
      insertItem(* YellowPixmap);
      break;
    case UmlMediumYellow:
      insertItem(* MediumYellowPixmap);
      break;
    case UmlDarkYellow:
      insertItem(* DarkYellowPixmap);
      break;
    case UmlLightBlue:
      insertItem(* LightBluePixmap);
      break;
    case UmlBlue:
      insertItem(* BluePixmap);
      break;
    case UmlMediumBlue:
      insertItem(* MediumBluePixmap);
      break;
    case UmlDarkBlue:
      insertItem(* DarkBluePixmap);
      break;
    case UmlLightGreen:
      insertItem(* LightGreenPixmap);
      break;
    case UmlGreen:
      insertItem(* GreenPixmap);
      break;
    case UmlMediumGreen:
      insertItem(* MediumGreenPixmap);
      break;
    case UmlDarkGreen:
      insertItem(* DarkGreenPixmap);
      break;
    case UmlRed:
      insertItem(* RedPixmap);
      break;
    case UmlLightGray:
      insertItem(* LightGrayPixmap);
      break;
    case UmlGray:
      insertItem(* GrayPixmap);
      break;
    default:
      insertItem("unknown color");
    }
  }
  if (!nodefault)
    insertItem(stringify(UmlDefaultColor));
  setSizeLimit(13);	// yes 13 !, don't set it to count() !
  setCurrentItem(v);
}

QSize SettingsDialog::previous_size;

SettingsDialog::SettingsDialog(QArray<StateSpec> * st, QArray<ColorSpec> * co,
			       bool nodefault, bool own)
    : QTabDialog(0, "Diagram Drawing Settings dialog", TRUE), states(st), colors(co) {
  setCaption("Diagram Drawing Settings dialog");
  
  QGrid * grid = 0;
  QString tabname;
  unsigned i;
  unsigned n;
  
  if (states != 0) {
    n = states->count();
    cbstates = new QVector<ComboStates>(n);
    
    for (i = 0; i != n; i += 1) {
      StateSpec & st = states->at(i);
      QString s = st.name;
      int index = s.find('#');
      QString tbn;
      
      if (index != -1) {
	tbn = s.left(index);
	s = s.mid(index + 1);
      }
      else
	tbn = "diagram";
      
      if ((grid == 0) || (tabname != tbn)) {
	if (grid != 0)
	  addTab(grid, tabname);
	grid = new QGrid(5, this);
	grid->setMargin(10);
	grid->setSpacing(10);
	tabname = tbn;
      }
      
      new QLabel("", grid);
      new QLabel(s + " : ", grid);
      QHBox * hb = new QHBox(grid);
            
      switch (st.who) {
      case StateSpec::is3states:
	cbstates->insert(i, new ComboStates(hb, *((Uml3States *) st.state), nodefault));
	break;
      case StateSpec::isClassDrawingMode:
	cbstates->insert(i, new ComboStates(hb, *((ClassDrawingMode *) st.state), nodefault));
	break;
      case StateSpec::isDrawingLanguage:
	cbstates->insert(i, new ComboStates(hb, *((DrawingLanguage *) st.state), nodefault));
	break;
      default:
	cbstates->insert(i, new ComboStates(hb, *((ShowContextMode *) st.state), nodefault));
      }
      new QLabel("", hb);
      new QLabel("", grid);
      new QLabel("", grid);
    }
    
    addTab(grid, tabname);
  }
  
  if (colors != 0) {
    const char * lbl = "color";
    
    n = colors->count();
    cbcolors = new QVector<ComboColor>(n);
    grid = new QGrid(5, this);
    grid->setMargin(10);
    grid->setSpacing(10);
    
    for (i = 0; i != n; i += 1) {
      if (i == 11) {
	addTab(grid, "color [1]");
	lbl = "color [2]";
	grid = new QGrid(5, this);
	grid->setMargin(10);
	grid->setSpacing(10);
      }
      new QLabel("", grid);
      QString s = (own) ? "" : "default ";
      s += colors->at(i).name;
      s += " : ";
      new QLabel(s, grid);
      cbcolors->insert(i, new ComboColor(grid, *(colors->at(i).color), nodefault));
      new QLabel("", grid);
      new QLabel("", grid);
    }
    
    addTab(grid, lbl);
  }
  
  setCancelButton();

  UmlDesktop::limitsize_center(this, previous_size, 0.8, 0.8);
}

SettingsDialog::~SettingsDialog() {
  if (colors != 0)
    delete cbcolors;
  if (states != 0)
    delete cbstates;
  previous_size = size();
}

void SettingsDialog::accept() {
  unsigned i, n;
  
  if (states != 0) {
    n = states->count();
    
    for (i = 0; i != n; i += 1)
      states->at(i).set_state(cbstates->at(i)->currentItem());
  }
  
  if (colors != 0) {
    n = colors->count();
    
    for (i = 0; i != n; i += 1)
      *(colors->at(i).color) = (UmlColor) cbcolors->at(i)->currentItem();
  }

  QDialog::accept();
}
