// *************************************************************************
//
// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
//
// This file is part of the BOUML Uml Toolkit.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// e-mail : bouml@free.fr
// home   : http://bouml.free.fr
//
// *************************************************************************

#ifdef WIN32
#pragma warning (disable: 4150)
#endif

#include <qlayout.h> 
#include <qpushbutton.h> 

#include "ToolDialog.h"
#include "Tool.h"
#include "UmlPixmap.h"
#include "UmlDesktop.h"

ToolDialog::ToolDialog(const QPoint &) : QDialog(0, "Tools dialog", TRUE) {
  setCaption("Tools dialog");
  QVBoxLayout * vbox = new QVBoxLayout(this);  
  
  //vbox->setMargin(5);
  
  QHBoxLayout * hbox = new QHBoxLayout(vbox); 
  //hbox->setMargin(5);
  
  table = new ToolTable(this);
  hbox->addWidget(table);
  
  hbox = new QHBoxLayout(vbox); 
  //hbox->setMargin(5);
  QPushButton * ok = new QPushButton("&OK", this);
  QPushButton * cancel = new QPushButton("&Cancel", this);
  QSize bs(cancel->sizeHint());
  
  ok->setDefault(TRUE);
  ok->setFixedSize(bs);
  cancel->setFixedSize(bs);
  
  hbox->addWidget(ok);
  hbox->addWidget(cancel);
  
  int w = (UmlDesktop::width() * 29) / 30;
  int h = (UmlDesktop::height() * 4) / 5;
  int dh = (table->numRows() + 5) *  table->rowHeight(0);
  
  resize(w, (dh > h) ? h : dh);
  if (UmlDesktop::fixed())
    UmlDesktop::tocenter(this);
  
  connect(ok, SIGNAL(clicked()), this, SLOT(accept()));
  connect(cancel, SIGNAL(clicked()), this, SLOT(reject()));
}

void ToolDialog::accept() {
  table->update();
  
  QDialog::accept();
}

// ToolTable

static const struct {
  UmlCode kind;
  QPixmap ** pixmap;
} ColDef[] = {
  { UmlPackage, &PackageIcon },
  { UmlUseCaseView, &UseCaseViewIcon },
  { UmlClassView, &ClassViewIcon },
  { UmlComponentView,&ComponentViewIcon },
  { UmlDeploymentView,&DeploymentViewIcon },
  { UmlUseCaseDiagram, &UseCaseDiagramIcon },
  { UmlSeqDiagram, &SeqDiagramIcon },
  { UmlColDiagram, &ColDiagramIcon },
  { UmlClassDiagram, &ClassDiagramIcon },
  { UmlObjectDiagram, &ObjectDiagramIcon },
  { UmlStateDiagram, &StateDiagramIcon },
  { UmlComponentDiagram,&ComponentDiagramIcon },
  { UmlDeploymentDiagram,&DeploymentDiagramIcon },
  { UmlUseCase, &usecaseButton },
  { UmlClass, &classButton },
  { UmlState, &StateIcon },
  { UmlComponent, &ComponentIcon },
  { UmlDeploymentNode, &DeploymentNodeIcon },
  { UmlArtifact, &ArtifactIcon },
  { UmlOperation, &PublicOperationIcon },
  { UmlAttribute, &PublicAttributeIcon },
  { UmlGeneralisation, &generalisationButton },
  { UmlRealize, &realizeButton },
  { UmlDependency, &dependencyButton },
  { UmlAssociation, &associationButton },
  { UmlDirectionalAssociation, &directionalAssociationButton },
  { UmlAggregation, &aggregationButton },
  { UmlAggregationByValue, &aggregationByValueButton },
  { UmlDirectionalAggregation, &directionalAggregationButton },
  { UmlDirectionalAggregationByValue, &directionalAggregationByValueButton },
  { UmlExtraMember, &ExtraMemberIcon },
  { UmlRegion, &RegionIcon },
  { InitialPS, &InitialIcon },
  { EntryPointPS, &EntryPointIcon },
  { FinalPS, &FinalIcon },
  { TerminatePS, &TerminateIcon },
  { ExitPointPS, &ExitPointIcon },
  { DeepHistoryPS, &DeepHistoryIcon },
  { ShallowHistoryPS, &ShallowHistoryIcon }, 
  { JunctionPS, &JunctionIcon },
  { ChoicePS, &ChoiceIcon },
  { ForkPS, &ForkIcon },
  { JoinPS, &JoinIcon },
  { UmlStateAction, &ActionIcon }
};

#define NCOL (sizeof(ColDef)/sizeof(ColDef[0]) + 4)

ToolTable::ToolTable(QWidget * parent)
    : StringTable(Tool::ntools + 1, NCOL, parent, FALSE) {
  
  horizontalHeader()->setLabel(0, "executable");
  horizontalHeader()->setLabel(1, "display");
  setColumnStretchable(0, FALSE);
  setColumnStretchable(1, FALSE);
  
  horizontalHeader()->setLabel(2, "Prj");
  adjustColumn(2);
  setColumnStretchable(2, FALSE);
  
  unsigned index;

  for (index = 3; index != NCOL-1; index += 1) {
    horizontalHeader()->setLabel(index, QIconSet(**(ColDef[index - 3].pixmap)),
				 "", 22/*24*/);
    setColumnStretchable(index, FALSE);
  }
  
  horizontalHeader()->setLabel(NCOL-1, "do");
  adjustColumn(NCOL-1);
  setColumnStretchable(NCOL-1, FALSE);
  
  unsigned rank;
  
  for (rank = 0; rank != Tool::ntools; rank += 1) {
    ATool & tool = Tool::tools[rank];
    
    setText(rank, 0, tool.cmd);
    setText(rank, 1, tool.display);
    
    if (tool.applicable[UmlProject])
      setText(rank, 2, " X");
    
    for (index = 3; index != NCOL-1; index += 1)
      if (tool.applicable[ColDef[index - 3].kind])
	setText(rank, index, " X");
  }
  init_row(rank);
  
  adjustColumn(0);
  adjustColumn(1);
}
  
void ToolTable::init_row(int row) {
  int index;
  
  for (index = 0; index != NCOL; index += 1)
    setText(row, index, QString::null);
}

void ToolTable::button_pressed(int row, int col, int b, const QPoint & p) {
  if ((col >= 2) && (col != (NCOL-1))) {
    setText(row, col, text(row, col).isEmpty() ? " X" : "");
  }
  else
    StringTable::button_pressed(row,  col, b, p);
}

void ToolTable::update() {
  forceUpdateCells();
  
  int n = numRows();
  
  if (text(n - 1, 0).isEmpty() || text(n - 1, 1).isEmpty())
    n -= 1;
  
  Tool::set_ntools(n);
  
  for (unsigned rank = 0; rank != Tool::ntools; rank += 1) {
    ATool & tool = Tool::tools[rank];
    
    tool.cmd = text(rank, 0).stripWhiteSpace();
    tool.display = text(rank, 1).stripWhiteSpace();
    
    tool.applicable[UmlProject] =
      !text(rank, 2).stripWhiteSpace().isEmpty();
    
    for (unsigned index = 3; index != NCOL-1; index += 1)
      tool.applicable[ColDef[index - 3].kind] =
	!text(rank, index).stripWhiteSpace().isEmpty();
  }  
}
